enum VisitMutAction<T> {
Enter(*mut T),
Leave(*mut T),
}Expand description
Frames for mutable post-traversals, will be kept in a stack.
Notice that we use mutable pointers, because mutable post-traversal is unsafe in rust.
The core argument for correctness mirrors the tree-borrow correctness argument Rust’s borrow checker uses for the ordinary function call stack. Loosely:
- We split a mutable parent node into its children, and put them on the stack.
- We keep a mutable pointer to the parent node, but won’t touch it until we’re done with all children.
- In the function call stack, this mutable pointer is the stack frame, safely inaccessible.
- In our
unsafeaction stack, this mutable pointer is below all of theEnteractions, and we promise not to touch it until they complete.
- When we have processed all children, we can reassemble access to the parent from its parts.
- In the function call stack, we do this on return from recursive calls.
- In our
unsafeaction stack, we do this after popping all children.
Variants
Enter(*mut T)
Put on the stack when entering a node.
Causes us to push children.
Leave(*mut T)
Put on the stack when leaving a node, all children visited.
Causes us to do the post-traversal visit of the parent.